@@ -50,21 +50,9 @@ class CliResourceList(lister.Lister):
     def take_action(self, parsed_args):
         resources = utils.get_client(self).resource.list(
             resource_type=parsed_args.resource_type,
-            **self._get_pagination_options(parsed_args))
+            **utils.get_pagination_options(parsed_args))
         return utils.list2cols(self.COLS, resources)
 
-    @staticmethod
-    def _get_pagination_options(parsed_args):
-        options = dict(
-            details=parsed_args.details,
-            sorts=parsed_args.sort,
-            limit=parsed_args.limit,
-            marker=parsed_args.marker)
-
-        if hasattr(parsed_args, 'history'):
-            options['history'] = parsed_args.history
-        return options
-
 
 class CliResourceHistory(CliResourceList):
     """Show the history of a resource"""
@@ -80,8 +68,11 @@ class CliResourceHistory(CliResourceList):
         resources = utils.get_client(self).resource.history(
             resource_type=parsed_args.resource_type,
             resource_id=parsed_args.resource_id,
-            **self._get_pagination_options(parsed_args))
-        return utils.list2cols(self.COLS, resources)
+            **utils.get_pagination_options(parsed_args))
+        cols = resources[0].keys() if resources else self.COLS
+        if parsed_args.formatter == 'table':
+            return utils.list2cols(cols, map(normalize_metrics, resources))
+        return utils.list2cols(cols, resources)
 
 
 class CliResourceSearch(CliResourceList):
@@ -96,7 +87,7 @@ class CliResourceSearch(CliResourceList):
         resources = utils.get_client(self).resource.search(
             resource_type=parsed_args.resource_type,
             query=parsed_args.query,
-            **self._get_pagination_options(parsed_args))
+            **utils.get_pagination_options(parsed_args))
         return utils.list2cols(self.COLS, resources)
 
 
@@ -104,6 +95,7 @@ def normalize_metrics(res):
     res['metrics'] = "\n".join(sorted(
         ["%s: %s" % (name, _id)
             for name, _id in res['metrics'].items()]))
+    return res
 
 
 class CliResourceShow(show.ShowOne):
@@ -121,7 +113,8 @@ class CliResourceShow(show.ShowOne):
         res = utils.get_client(self).resource.get(
             resource_type=parsed_args.resource_type,
             resource_id=parsed_args.resource_id)
-        normalize_metrics(res)
+        if parsed_args.formatter == 'table':
+            normalize_metrics(res)
         return self.dict2columns(res)
 
 
@@ -136,7 +129,7 @@ class CliResourceCreate(show.ShowOne):
                             help="ID of the resource")
         parser.add_argument("-a", "--attribute", action='append',
                             default=[],
-                            help=("name and value of a attribute "
+                            help=("name and value of an attribute "
                                   "separated with a ':'"))
         parser.add_argument("-m", "--add-metric", action='append',
                             default=[],
@@ -187,7 +180,8 @@ class CliResourceCreate(show.ShowOne):
         resource = self._resource_from_args(parsed_args)
         res = utils.get_client(self).resource.create(
             resource_type=parsed_args.resource_type, resource=resource)
-        normalize_metrics(res)
+        if parsed_args.formatter == 'table':
+            normalize_metrics(res)
         return self.dict2columns(res)
 
 
@@ -207,7 +201,8 @@ class CliResourceUpdate(CliResourceCreate):
             resource_type=parsed_args.resource_type,
             resource_id=parsed_args.resource_id,
             resource=resource)
-        normalize_metrics(res)
+        if parsed_args.formatter == 'table':
+            normalize_metrics(res)
         return self.dict2columns(res)
 
 
@@ -222,14 +217,3 @@ class CliResourceDelete(command.Command):
 
     def take_action(self, parsed_args):
         utils.get_client(self).resource.delete(parsed_args.resource_id)
-
-
-class CliResourceTypeList(lister.Lister):
-    """List the resource types that gnocchi supports"""
-
-    COLS = ('resource_type',
-            'resource_controller_url')
-
-    def take_action(self, parsed_args):
-        resources = utils.get_client(self).resource.list_types()
-        return self.COLS, list(resources.items())
